@@ -52,8 +52,6 @@
 #include "notificationdaemon-dbus-glue.h"
 
 #define MAX_NOTIFICATIONS 20
-
-#define WIDTH 400
 #define IMAGE_SIZE 48
 #define IDLE_SECONDS 30
 #define NOTIFICATION_BUS_NAME      "org.freedesktop.Notifications"
@@ -108,8 +106,12 @@ struct _NotifyDaemonPrivate {
 	gboolean url_clicked_lock;
 
 	NotifyStackLocation stack_location;
+#if GTK_CHECK_VERSION (3, 8, 0)
+	NotifyScreen* screen;
+#else
 	NotifyScreen** screens;
 	int n_screens;
+#endif
 };
 
 typedef struct {
@@ -117,7 +119,7 @@ typedef struct {
 	NotifyDaemon* daemon;
 } _NotifyPendingClose;
 
-static DBusConnection* dbus_conn = NULL;
+static DBusConnection* dbus_conn;
 
 static void notify_daemon_finalize(GObject* object);
 static void _notification_destroyed_cb(GtkWindow* nw, NotifyDaemon* daemon);
@@ -125,38 +127,18 @@ static void _close_notification(NotifyDaemon* daemon, guint id, gboolean hide_no
 static GdkFilterReturn _notify_x11_filter(GdkXEvent* xevent, GdkEvent* event, NotifyDaemon* daemon);
 static void _emit_closed_signal(GtkWindow* nw, NotifydClosedReason reason);
 static void _action_invoked_cb(GtkWindow* nw, const char* key);
-static NotifyStackLocation get_stack_location_from_string(const char* slocation);
+static NotifyStackLocation get_stack_location_from_string(const gchar *slocation);
 static void sync_notification_position(NotifyDaemon* daemon, GtkWindow* nw, Window source);
 static void monitor_notification_source_windows(NotifyDaemon* daemon, NotifyTimeout* nt, Window source);
 
 G_DEFINE_TYPE(NotifyDaemon, notify_daemon, G_TYPE_OBJECT);
 
-#if GTK_CHECK_VERSION (3, 0, 0)
-static void
-notify_daemon_get_preferred_width (GtkWidget *widget,
-                                   gint *min_width,
-                                   gint *nat_width)
-{
-        if (nat_width != NULL) {
-            *nat_width = WIDTH;
-        }
-}
-#endif
-
 static void notify_daemon_class_init(NotifyDaemonClass* daemon_class)
 {
 	GObjectClass* object_class = G_OBJECT_CLASS(daemon_class);
 
-#if GTK_CHECK_VERSION (3, 0, 0)
-    GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (daemon_class);
-#endif
-
 	object_class->finalize = notify_daemon_finalize;
 
-#if GTK_CHECK_VERSION (3, 0, 0)
-    widget_class->get_preferred_width = notify_daemon_get_preferred_width;
-#endif
-
 	g_type_class_add_private(daemon_class, sizeof(NotifyDaemonPrivate));
 }
 
@@ -201,8 +183,12 @@ static void remove_exit_timeout(NotifyDaemon* daemon)
 
 static void create_stack_for_monitor(NotifyDaemon* daemon, GdkScreen* screen, int monitor_num)
 {
+#if GTK_CHECK_VERSION (3, 8, 0)
+	NotifyScreen* nscreen = daemon->priv->screen;
+#else
 	int screen_num = gdk_screen_get_number(screen);
 	NotifyScreen* nscreen = daemon->priv->screens[screen_num];
+#endif
 
 	nscreen->stacks[monitor_num] = notify_stack_new(daemon, screen, monitor_num, daemon->priv->stack_location);
 }
@@ -210,12 +196,18 @@ static void create_stack_for_monitor(NotifyDaemon* daemon, GdkScreen* screen, in
 static void on_screen_monitors_changed(GdkScreen* screen, NotifyDaemon* daemon)
 {
 	NotifyScreen* nscreen;
+#if !GTK_CHECK_VERSION (3, 8, 0)
 	int screen_num;
+#endif
 	int n_monitors;
 	int i;
 
+#if GTK_CHECK_VERSION (3, 8, 0)
+	nscreen = daemon->priv->screen;
+#else
 	screen_num = gdk_screen_get_number(screen);
 	nscreen = daemon->priv->screens[screen_num];
+#endif
 
 	n_monitors = gdk_screen_get_n_monitors(screen);
 
@@ -267,11 +259,17 @@ static void on_screen_monitors_changed(GdkScreen* screen, NotifyDaemon* daemon)
 static void create_stacks_for_screen(NotifyDaemon* daemon, GdkScreen *screen)
 {
 	NotifyScreen* nscreen;
+#if !GTK_CHECK_VERSION (3, 8, 0)
 	int screen_num;
+#endif
 	int i;
 
+#if GTK_CHECK_VERSION (3, 8, 0)
+	nscreen = daemon->priv->screen;
+#else
 	screen_num = gdk_screen_get_number(screen);
 	nscreen = daemon->priv->screens[screen_num];
+#endif
 
 	nscreen->n_stacks = gdk_screen_get_n_monitors(screen);
 
@@ -300,6 +298,31 @@ static GdkFilterReturn screen_xevent_filter(GdkXEvent* xevent, GdkEvent* event,
 	return GDK_FILTER_CONTINUE;
 }
 
+#if GTK_CHECK_VERSION (3, 8, 0)
+static void create_screen(NotifyDaemon* daemon)
+{
+    GdkDisplay *display;
+    GdkScreen  *screen;
+    GdkWindow  *gdkwindow;
+
+	g_assert(daemon->priv->screen == NULL);
+
+	display = gdk_display_get_default();
+	screen = gdk_display_get_default_screen (display);
+
+	g_signal_connect(screen, "monitors-changed", G_CALLBACK(on_screen_monitors_changed), daemon);
+
+	daemon->priv->screen = g_new0(NotifyScreen, 1);
+
+	daemon->priv->screen->workarea_atom = XInternAtom(GDK_DISPLAY_XDISPLAY (display), "_NET_WORKAREA", True);
+
+	gdkwindow = gdk_screen_get_root_window(screen);
+	gdk_window_add_filter(gdkwindow, (GdkFilterFunc) screen_xevent_filter, daemon->priv->screen);
+	gdk_window_set_events(gdkwindow, gdk_window_get_events(gdkwindow) | GDK_PROPERTY_CHANGE_MASK);
+
+	create_stacks_for_screen(daemon, screen);
+}
+#else
 static void create_screens(NotifyDaemon* daemon)
 {
 	GdkDisplay* display;
@@ -331,11 +354,12 @@ static void create_screens(NotifyDaemon* daemon)
 		create_stacks_for_screen(daemon, gdk_display_get_screen(display, i));
 	}
 }
+#endif
 
 static void on_popup_location_changed(GSettings *settings, gchar *key, NotifyDaemon* daemon)
 {
 	NotifyStackLocation stack_location;
-	const char* slocation;
+	gchar *slocation;
 	int i;
 
 	slocation = g_settings_get_string(daemon->gsettings, key);
@@ -352,7 +376,19 @@ static void on_popup_location_changed(GSettings *settings, gchar *key, NotifyDae
 	}
 
 	daemon->priv->stack_location = stack_location;
+	g_free(slocation);
 
+#if GTK_CHECK_VERSION (3, 8, 0)
+    NotifyScreen *nscreen;
+
+	nscreen = daemon->priv->screen;
+	for (i = 0; i < nscreen->n_stacks; i++)
+	{
+		NotifyStack* stack;
+		stack = nscreen->stacks[i];
+		notify_stack_set_location(stack, stack_location);
+	}
+#else
 	for (i = 0; i < daemon->priv->n_screens; i++)
 	{
 		int j;
@@ -364,11 +400,12 @@ static void on_popup_location_changed(GSettings *settings, gchar *key, NotifyDae
 			notify_stack_set_location(stack, stack_location);
 		}
 	}
+#endif
 }
 
 static void notify_daemon_init(NotifyDaemon* daemon)
 {
-	char* location;
+	gchar *location;
 
 	daemon->priv = G_TYPE_INSTANCE_GET_PRIVATE(daemon, NOTIFY_TYPE_DAEMON, NotifyDaemonPrivate);
 
@@ -385,16 +422,50 @@ static void notify_daemon_init(NotifyDaemon* daemon)
 	daemon->priv->stack_location = get_stack_location_from_string(location);
 	g_free(location);
 
+#if GTK_CHECK_VERSION (3, 8, 0)
+	daemon->priv->screen = NULL;
+
+	create_screen(daemon);
+#else
 	daemon->priv->n_screens = 0;
 	daemon->priv->screens = NULL;
 
 	create_screens(daemon);
+#endif
 
 	daemon->priv->idle_reposition_notify_ids = g_hash_table_new(NULL, NULL);
 	daemon->priv->monitored_window_hash = g_hash_table_new(NULL, NULL);
 	daemon->priv->notification_hash = g_hash_table_new_full(g_int_hash, g_int_equal, g_free, (GDestroyNotify) _notify_timeout_destroy);
 }
 
+#if GTK_CHECK_VERSION (3, 8, 0)
+static void destroy_screen(NotifyDaemon* daemon)
+{
+	GdkDisplay *display;
+	GdkScreen  *screen;
+	GdkWindow  *gdkwindow;
+	gint        i;
+
+	display = gdk_display_get_default();
+	screen = gdk_display_get_default_screen (display);
+
+	g_signal_handlers_disconnect_by_func (screen,
+										  G_CALLBACK (on_screen_monitors_changed),
+										  daemon);
+
+	gdkwindow = gdk_screen_get_root_window (screen);
+	gdk_window_remove_filter (gdkwindow, (GdkFilterFunc) screen_xevent_filter, daemon->priv->screen);
+	for (i = 0; i < daemon->priv->screen->n_stacks; i++) {
+		 g_clear_object (&daemon->priv->screen->stacks[i]);
+	}
+
+	g_free (daemon->priv->screen->stacks);
+	daemon->priv->screen->stacks = NULL;
+
+	g_free(daemon->priv->screen);
+	daemon->priv->screen = NULL;
+}
+#else
 static void destroy_screens(NotifyDaemon* daemon)
 {
 	GdkDisplay* display;
@@ -426,6 +497,7 @@ static void destroy_screens(NotifyDaemon* daemon)
 	g_free(daemon->priv->screens);
 	daemon->priv->screens = NULL;
 }
+#endif
 
 static void notify_daemon_finalize(GObject* object)
 {
@@ -444,14 +516,18 @@ static void notify_daemon_finalize(GObject* object)
 	g_hash_table_destroy(daemon->priv->idle_reposition_notify_ids);
 	g_hash_table_destroy(daemon->priv->notification_hash);
 
+#if GTK_CHECK_VERSION (3, 8, 0)
+	destroy_screen(daemon);
+#else
 	destroy_screens(daemon);
+#endif
 
 	g_free(daemon->priv);
 
 	G_OBJECT_CLASS(notify_daemon_parent_class)->finalize(object);
 }
 
-static NotifyStackLocation get_stack_location_from_string(const char* slocation)
+static NotifyStackLocation get_stack_location_from_string(const gchar *slocation)
 {
 	NotifyStackLocation stack_location = NOTIFY_STACK_LOCATION_DEFAULT;
 
@@ -1025,7 +1101,11 @@ static GdkPixbuf* _notify_daemon_pixbuf_from_path(const char* path)
 
 			pixbuf = gtk_icon_theme_load_icon (theme, path, icon_size, GTK_ICON_LOOKUP_USE_BUILTIN, NULL);
 
+#if GTK_CHECK_VERSION (3, 8, 0)
+			g_object_unref (icon_info);
+#else
 			gtk_icon_info_free (icon_info);
+#endif
 		}
 
 		if (pixbuf == NULL)
@@ -1042,25 +1122,13 @@ static GdkPixbuf* _notify_daemon_scale_pixbuf(GdkPixbuf *pixbuf, gboolean no_str
 {
 	int pw;
 	int ph;
-	float scale_factor_x = 1.0;
-	float scale_factor_y = 1.0;
-	float scale_factor = 1.0;
+	float scale_factor;
 
 	pw = gdk_pixbuf_get_width (pixbuf);
 	ph = gdk_pixbuf_get_height (pixbuf);
 
 	/* Determine which dimension requires the smallest scale. */
-	scale_factor_x = (float) IMAGE_SIZE / (float) pw;
-	scale_factor_y = (float) IMAGE_SIZE / (float) ph;
-
-	if (scale_factor_x > scale_factor_y)
-	{
-		scale_factor = scale_factor_y;
-	}
-	else
-	{
-		scale_factor = scale_factor_x;
-	}
+	scale_factor = (float) IMAGE_SIZE / (float) MAX(pw, ph);
 
 	/* always scale down, allow to disable scaling up */
 	if (scale_factor < 1.0 || !no_stretch_hint)
@@ -1313,7 +1381,7 @@ static void sync_notification_position(NotifyDaemon* daemon, GtkWindow* nw, Wind
 
 GQuark notify_daemon_error_quark(void)
 {
-	static GQuark q = 0;
+	static GQuark q;
 
 	if (q == 0)
 	{
@@ -1524,7 +1592,14 @@ gboolean notify_daemon_notify_handler(NotifyDaemon* daemon, const char* app_name
 	else
 	{
 		int monitor_num;
+#if !GTK_CHECK_VERSION (3, 8, 0)
 		int screen_num;
+#endif
+#if GTK_CHECK_VERSION (3, 0, 0)
+		GdkDisplay *display;
+		GdkDeviceManager *device_manager;
+		GdkDevice *pointer;
+#endif
 		GdkScreen* screen;
 		gint x, y;
 
@@ -1535,25 +1610,49 @@ gboolean notify_daemon_notify_handler(NotifyDaemon* daemon, const char* app_name
 		 * number the user has set in gsettings. */
 		if (g_settings_get_boolean(daemon->gsettings, GSETTINGS_KEY_USE_ACTIVE))
 		{
+#if GTK_CHECK_VERSION (3, 0, 0)
+			display = gdk_display_get_default ();
+			device_manager = gdk_display_get_device_manager (display);
+			pointer = gdk_device_manager_get_client_pointer (device_manager);
+
+			gdk_device_get_position (pointer, &screen, &x, &y);
+#else
 			gdk_display_get_pointer (gdk_display_get_default (), &screen, &x, &y, NULL);
+#endif
+#if !GTK_CHECK_VERSION (3, 8, 0)
 			screen_num = gdk_screen_get_number (screen);
+#endif
 			monitor_num = gdk_screen_get_monitor_at_point (screen, x, y);
 		}
 		else
 		{
 			screen = gdk_display_get_default_screen(gdk_display_get_default());
+#if !GTK_CHECK_VERSION (3, 8, 0)
 			screen_num = gdk_screen_get_number(screen);
+#endif
 			monitor_num = g_settings_get_int(daemon->gsettings, GSETTINGS_KEY_MONITOR_NUMBER);
 		}
 
+#if GTK_CHECK_VERSION (3, 8, 0)
+		if (monitor_num >= priv->screen->n_stacks)
+#else
 		if (monitor_num >= priv->screens[screen_num]->n_stacks)
+#endif
 		{
 			/* screw it - dump it on the last one we'll get
 			 a monitors-changed signal soon enough*/
+#if GTK_CHECK_VERSION (3, 8, 0)
+			monitor_num = priv->screen->n_stacks - 1;
+#else
 			monitor_num = priv->screens[screen_num]->n_stacks - 1;
+#endif
 		}
 
+#if GTK_CHECK_VERSION (3, 8, 0)
+		notify_stack_add_window (priv->screen->stacks[monitor_num], nw, new_notification);
+#else
 		notify_stack_add_window (priv->screens[screen_num]->stacks[monitor_num], nw, new_notification);
+#endif
 	}
 
 	if (id == 0)
@@ -1668,7 +1767,7 @@ int main(int argc, char** argv)
 	gboolean res;
 	guint request_name_result;
 
-	g_log_set_always_fatal(G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL);
+	g_log_set_always_fatal(G_LOG_LEVEL_CRITICAL);
 
 	gtk_init(&argc, &argv);
 
