@@ -62,10 +62,12 @@ typedef struct
 
 	ArrowParameters arrow;
 
-	gboolean enable_transparency;
+	gboolean composited;
 	
 	int width;
 	int height;
+	int last_width;
+	int last_height;
 
 	guchar urgency;
 	glong timeout;
@@ -83,7 +85,23 @@ enum
 	URGENCY_CRITICAL
 };
 
-#define M_PI 3.14159265358979323846
+gboolean theme_check_init(unsigned int major_ver, unsigned int minor_ver,
+			  unsigned int micro_ver);
+void get_theme_info(char **theme_name, char **theme_ver, char **author,
+		    char **homepage);
+GtkWindow* create_notification(UrlClickedCb url_clicked);
+void set_notification_text(GtkWindow *nw, const char *summary,
+			   const char *body);
+void set_notification_icon(GtkWindow *nw, GdkPixbuf *pixbuf);
+void set_notification_arrow(GtkWidget *nw, gboolean visible, int x, int y);
+void add_notification_action(GtkWindow *nw, const char *text, const char *key,
+			     ActionInvokedCb cb);
+void clear_notification_actions(GtkWindow *nw);
+void move_notification(GtkWidget *nw, int x, int y);
+void set_notification_timeout(GtkWindow *nw, glong timeout);
+void set_notification_hints(GtkWindow *nw, GHashTable *hints);
+void notification_tick(GtkWindow *nw, glong remaining);
+
 #define STRIPE_WIDTH  32
 #define WIDTH         400
 #define IMAGE_SIZE    32
@@ -100,6 +118,11 @@ enum
 #define BACKGROUND_OPACITY    0.92
 #define GRADIENT_CENTER 0.7
 
+#if GTK_CHECK_VERSION (3, 2, 0)
+#define gtk_hbox_new(X,Y) gtk_box_new(GTK_ORIENTATION_HORIZONTAL,Y)
+#define gtk_vbox_new(X,Y) gtk_box_new(GTK_ORIENTATION_VERTICAL,Y)
+#endif
+
 /* Support Nodoka Functions */
 
 /* Handle clicking on link */
@@ -135,13 +158,10 @@ get_notification_arrow_type(GtkWidget *nw)
 static void
 set_arrow_parameters (WindowData *windata)
 {
-	int screen_height;
 	int screen_width;
 	int x,y;
 	GtkArrowType arrow_type;
 
-	screen_height = gdk_screen_get_height(
-		gdk_window_get_screen(gtk_widget_get_window(windata->win)));
 	screen_width = gdk_screen_get_width(
 		gdk_window_get_screen(gtk_widget_get_window(windata->win)));
 
@@ -283,10 +303,10 @@ nodoka_rounded_rectangle (cairo_t * cr,
 							  int radius)
 {
 	cairo_move_to (cr, x + radius, y);
-	cairo_arc (cr, x + w - radius, y + radius, radius, M_PI * 1.5, M_PI * 2);
-	cairo_arc (cr, x + w - radius, y + h - radius, radius, 0, M_PI * 0.5);
-	cairo_arc (cr, x + radius, y + h - radius, radius, M_PI * 0.5, M_PI);
-	cairo_arc (cr, x + radius, y + radius, radius, M_PI, M_PI * 1.5);
+	cairo_arc (cr, x + w - radius, y + radius, radius, G_PI * 1.5, G_PI * 2);
+	cairo_arc (cr, x + w - radius, y + h - radius, radius, 0, G_PI * 0.5);
+	cairo_arc (cr, x + radius, y + h - radius, radius, G_PI * 0.5, G_PI);
+	cairo_arc (cr, x + radius, y + radius, radius, G_PI, G_PI * 1.5);
 }
 
 /* Rounded rectangle with arrow */
@@ -300,7 +320,7 @@ nodoka_rounded_rectangle_with_arrow (cairo_t * cr,
 
 	cairo_translate (cr, x, y);
 
-	GdkRectangle rect;
+	cairo_rectangle_int_t rect;
 	rect.x = 0;
 	rect.width = w;
 	if (arrow_up)
@@ -322,9 +342,9 @@ nodoka_rounded_rectangle_with_arrow (cairo_t * cr,
 	}
 
 	cairo_arc (cr, rect.x + rect.width - radius, rect.y + radius, radius, 
-			M_PI * 1.5, M_PI * 2);
+			G_PI * 1.5, G_PI * 2);
 	cairo_arc (cr, rect.x + rect.width - radius, 
-			rect.y + rect.height - radius, radius, 0, M_PI * 0.5);
+			rect.y + rect.height - radius, radius, 0, G_PI * 0.5);
 
 	if (!arrow_up)
 	{
@@ -337,9 +357,9 @@ nodoka_rounded_rectangle_with_arrow (cairo_t * cr,
 	}
 
 	cairo_arc (cr, rect.x + radius, rect.y + rect.height - radius, 
-			radius, M_PI * 0.5, M_PI);
-	cairo_arc (cr, rect.x + radius, rect.y + radius, radius, M_PI, 
-			M_PI * 1.5);
+			radius, G_PI * 0.5, G_PI);
+	cairo_arc (cr, rect.x + radius, rect.y + radius, radius, G_PI,
+			G_PI * 1.5);
 
 	cairo_translate (cr, -x, -y);
 
@@ -350,7 +370,7 @@ static void
 fill_background(GtkWidget *widget, WindowData *windata, cairo_t *cr)
 {
 	float alpha;
-	if (windata->enable_transparency)
+	if (windata->composited)
 		alpha = BACKGROUND_OPACITY;
 	else
 		alpha = 1.0;
@@ -380,12 +400,20 @@ draw_stripe(GtkWidget *widget, WindowData *windata, cairo_t *cr)
 	cairo_rectangle (cr, 0, 0, STRIPE_WIDTH, windata->height);
 	cairo_clip (cr);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gdouble  color_mult = 1.0;
+	GdkRGBA  top_color;
+	GdkRGBA  center_color;
+	GdkRGBA  bottom_color;
+#else
+	gdouble  color_mult = 65535.0;
 	GdkColor top_color;
 	GdkColor center_color;
 	GdkColor bottom_color;
+#endif
 
 	float alpha;
-	if (windata->enable_transparency)
+	if (windata->composited)
 		alpha = BACKGROUND_OPACITY;
 	else
 		alpha = 1.0;
@@ -394,49 +422,49 @@ draw_stripe(GtkWidget *widget, WindowData *windata, cairo_t *cr)
 	{
 		case URGENCY_LOW: // LOW
 			alpha = alpha * 0.5;
-			top_color.red = 221 / 255.0 * 65535.0;
+			top_color.red = 221 / 255.0 * color_mult;
 			top_color.green = top_color.red;
 			top_color.blue = top_color.red;
-			center_color.red = 192 / 255.0 * 65535.0;
+			center_color.red = 192 / 255.0 * color_mult;
 			center_color.green = center_color.red;
 			center_color.blue = center_color.red;
-			bottom_color.red = 167 / 255.0 * 65535.0;
+			bottom_color.red = 167 / 255.0 * color_mult;
 			bottom_color.green = center_color.red;
 			bottom_color.blue = center_color.red;
 			break;
 
 		case URGENCY_CRITICAL: // CRITICAL
-			top_color.red = 255 / 255.0 * 65535.0;
-			top_color.green = 11 / 255.0 * 65535.0;
+			top_color.red = 255 / 255.0 * color_mult;
+			top_color.green = 11 / 255.0 * color_mult;
 			top_color.blue = top_color.green;
-			center_color.red = 205 / 255.0 * 65535.0;
+			center_color.red = 205 / 255.0 * color_mult;
 			center_color.green = 0;
 			center_color.blue = 0;
-			bottom_color.red = 145 / 255.0 * 65535.0;
+			bottom_color.red = 145 / 255.0 * color_mult;
 			bottom_color.green = 0;
 			bottom_color.blue = 0;
 			break;
 
 		case URGENCY_NORMAL: // NORMAL
 		default:
-			top_color.red = 20 / 255.0 * 65535.0;
-			top_color.green = 175 / 255.0 * 65535.0;
-			top_color.blue = 65535.0;
-			center_color.red = 9 / 255.0 * 65535.0;
-			center_color.green = 139 / 255.0 * 65535.0;
-			center_color.blue = 207 / 255.0 * 65535.0;
+			top_color.red = 20 / 255.0 * color_mult;
+			top_color.green = 175 / 255.0 * color_mult;
+			top_color.blue = color_mult;
+			center_color.red = 9 / 255.0 * color_mult;
+			center_color.green = 139 / 255.0 * color_mult;
+			center_color.blue = 207 / 255.0 * color_mult;
 			bottom_color.red = 0;
-			bottom_color.green = 97 / 255.0 * 65535.0;
-			bottom_color.blue = 147 / 255.0 * 65535.0;
+			bottom_color.green = 97 / 255.0 * color_mult;
+			bottom_color.blue = 147 / 255.0 * color_mult;
 			break;
 	}
 
 
 	cairo_pattern_t *pattern;
 	pattern = cairo_pattern_create_linear (0, 0, 0, windata->height);
-	cairo_pattern_add_color_stop_rgba (pattern, 0, top_color.red / 65535.0, top_color.green / 65535.0, top_color.blue / 65535.0, alpha);
-	cairo_pattern_add_color_stop_rgba (pattern, GRADIENT_CENTER, bottom_color.red / 65535.0, bottom_color.green / 65535.0, bottom_color.blue / 65535.0, alpha);
-	cairo_pattern_add_color_stop_rgba (pattern, 1, bottom_color.red / 65535.0, bottom_color.green / 65535.0, bottom_color.blue / 65535.0, alpha);
+	cairo_pattern_add_color_stop_rgba (pattern, 0, top_color.red / color_mult, top_color.green / color_mult, top_color.blue / color_mult, alpha);
+	cairo_pattern_add_color_stop_rgba (pattern, GRADIENT_CENTER, bottom_color.red / color_mult, bottom_color.green / color_mult, bottom_color.blue / color_mult, alpha);
+	cairo_pattern_add_color_stop_rgba (pattern, 1, bottom_color.red / color_mult, bottom_color.green / color_mult, bottom_color.blue / color_mult, alpha);
 	cairo_set_source (cr, pattern);
 	cairo_pattern_destroy (pattern);
 
@@ -455,7 +483,7 @@ static void
 draw_border(GtkWidget *widget, WindowData *windata, cairo_t *cr)
 {
 	float alpha;
-	if (windata->enable_transparency)
+	if (windata->composited)
 		alpha = BACKGROUND_OPACITY;
 	else
 		alpha = 1.0;
@@ -488,118 +516,188 @@ draw_pie(GtkWidget *pie, WindowData *windata, cairo_t *cr)
 	cairo_set_source_rgba (cr, 1.0, 0.4, 0.0, 0.3);
 	cairo_move_to(cr, PIE_RADIUS, PIE_RADIUS);
 	cairo_arc_negative(cr, PIE_RADIUS, PIE_RADIUS, PIE_RADIUS,
-					-M_PI/2, (-0.25 + arc_angle)*2*M_PI);
+					-G_PI/2, (-0.25 + arc_angle)*2*G_PI);
 	cairo_line_to(cr, PIE_RADIUS, PIE_RADIUS);
 	
 	cairo_fill (cr); 
 }
 
-static gboolean
-paint_window(GtkWidget *widget,
 #if GTK_CHECK_VERSION (3, 0, 0)
-			 cairo_t *cr,
-#else
-			 GdkEventExpose *event,
-#endif
-			 WindowData *windata)
+static void
+update_shape_region (cairo_surface_t *surface,
+		     WindowData      *windata)
 {
-	cairo_t *context;
-	cairo_surface_t *surface;
-	GtkAllocation alloc;
-	gtk_widget_get_allocation(windata->win, &alloc);
-#if GTK_CHECK_VERSION(3, 0, 0)
-	cairo_t *cr2;
+	if (windata->width == windata->last_width && windata->height == windata->last_height)
+	{
+		return;
+	}
+
+	if (windata->width == 0 || windata->height == 0)
+	{
+		GtkAllocation allocation;
+		gtk_widget_get_allocation (windata->win, &allocation);
+
+		windata->width = MAX (allocation.width, 1);
+		windata->height = MAX (allocation.height, 1);
+	}
+
+	if (!windata->composited) {
+		cairo_region_t *region;
+
+		region = gdk_cairo_region_create_from_surface (surface);
+		gtk_widget_shape_combine_region (windata->win, region);
+		cairo_region_destroy (region);
+	} else {
+		gtk_widget_shape_combine_region (windata->win, NULL);
+		return;
+	}
+
+	windata->last_width = windata->width;
+	windata->last_height = windata->height;
+}
 #else
+static void
+update_shape_mask (WindowData* windata)
+{
 	cairo_t *cr;
+	GdkBitmap* mask;
+
+	if (windata->width == windata->last_width && windata->height == windata->last_height)
+	{
+		return;
+	}
+
+	if (windata->width == 0 || windata->height == 0)
+	{
+		GtkAllocation allocation;
+
+		gtk_widget_get_allocation (windata->win, &allocation);
+
+		windata->width = MAX (allocation.width, 1);
+		windata->height = MAX (allocation.height, 1);
+	}
+
+	if (windata->composited)
+	{
+		gtk_widget_shape_combine_mask (windata->win, NULL, 0, 0);
+		return;
+	}
+
+	windata->last_width = windata->width;
+	windata->last_height = windata->height;
+
+	mask = (GdkBitmap*) gdk_pixmap_new (NULL, windata->width, windata->height, 1);
+
+	if (mask == NULL)
+	{
+		return;
+	}
+	cr = gdk_cairo_create (mask);
+
+	if (cairo_status (cr) == CAIRO_STATUS_SUCCESS)
+	{
+		cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
+		cairo_paint (cr);
+
+		cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+		cairo_set_source_rgb (cr, 1.0f, 1.0f, 1.0f);
+		nodoka_rounded_rectangle (cr, 0, 0, windata->width , windata->height, 6);
+		cairo_fill (cr);
+
+		gtk_widget_shape_combine_mask (windata->win, mask, 0, 0);
+	}
+
+	cairo_destroy (cr);
+	g_object_unref (mask);
+}
 #endif
 
-	if (windata->width == 0) {
-		windata->width = alloc.width;
-		windata->height = alloc.height;
+static void
+paint_window (GtkWidget  *widget,
+	      cairo_t    *cr,
+	      WindowData *windata)
+{
+	cairo_t *cr2;
+	cairo_surface_t *surface;
+	GtkAllocation allocation;
+
+	if (windata->width == 0 || windata->height == 0) {
+		gtk_widget_get_allocation (windata->win, &allocation);
+		windata->width = allocation.width;
+		windata->height = allocation.height;
 	}
 	
 	if (windata->arrow.has_arrow)
 		set_arrow_parameters (windata);
 
-	if (!(windata->enable_transparency))
-	{
-#if GTK_CHECK_VERSION (3, 0, 0)
-			surface = cairo_surface_create_similar (cairo_get_target (cr),
-													CAIRO_CONTENT_COLOR_ALPHA,
-													windata->width,
-													windata->height);
+	surface = cairo_surface_create_similar (cairo_get_target (cr),
+						CAIRO_CONTENT_COLOR_ALPHA,
+						windata->width,
+						windata->height);
 
-/*
-			cr2 = cairo_create (surface);
-			/ * transparent background * /
-			cairo_rectangle (cr2, 0, 0, windata->width, windata->height);
-			cairo_set_source_rgba (cr2, 0.0, 0.0, 0.0, 0.0);
-			cairo_fill (cr2);
-			cairo_destroy (cr2);
-*/
+	cr2 = cairo_create (surface);
 
-			cairo_save (cr);
-			cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
-			cairo_set_source_surface (cr, surface, 0, 0);
-			cairo_paint (cr);
-			cairo_restore (cr);
-			cairo_surface_destroy(surface);
-#else
-			GdkPixmap *mask;
-			cairo_t *mask_cr;
-			mask = gdk_pixmap_new (NULL, windata->width, 
-						     windata->height, 1);
-			mask_cr = gdk_cairo_create ((GdkDrawable *) mask);
-			cairo_set_operator (mask_cr, CAIRO_OPERATOR_CLEAR);
-			cairo_paint (mask_cr);
+        /* transparent background */
+        cairo_rectangle (cr2, 0, 0, windata->width, windata->height);
+        cairo_set_source_rgba (cr2, 0.0, 0.0, 0.0, 0.0);
+        cairo_fill (cr2);
 
-			cairo_set_operator (mask_cr, CAIRO_OPERATOR_OVER);
-			cairo_set_source_rgba (mask_cr, 1, 1, 1, 1);
-			if (windata->arrow.has_arrow)
-			{
-				nodoka_rounded_rectangle_with_arrow (mask_cr, 0, 
-					0, windata->width, windata->height, 6, 
-					& (windata->arrow));
-			}
-			else
-				nodoka_rounded_rectangle (mask_cr, 0, 0, 
-					windata->width, windata->height, 6);
-			cairo_fill (mask_cr);
-			gdk_window_shape_combine_mask (windata->win->window,
-						       (GdkBitmap *) mask, 0,0);
-			gdk_pixmap_unref (mask);
-			cairo_destroy (mask_cr);
-#endif
+	if (windata->arrow.has_arrow) {
+		nodoka_rounded_rectangle_with_arrow (cr2, 0, 0,
+						     windata->width,
+						     windata->height,
+						     6,
+						     & (windata->arrow));
+	} else {
+		nodoka_rounded_rectangle (cr2, 0, 0,
+					  windata->width,
+					  windata->height,
+					  6);
 	}
 
-	context = gdk_cairo_create(gtk_widget_get_window(widget));
-	gtk_widget_get_allocation(widget, &alloc);
+	cairo_fill (cr2);
 
-	cairo_set_operator(context, CAIRO_OPERATOR_SOURCE);
-	surface = cairo_surface_create_similar(cairo_get_target(context),
-										   CAIRO_CONTENT_COLOR_ALPHA,
-										   alloc.width,
-										   alloc.height);
-#if GTK_CHECK_VERSION(3, 0, 0)
+	fill_background(widget, windata, cr2);
+	draw_border(widget, windata, cr2);
+	draw_stripe(widget, windata, cr2);
+
+	cairo_destroy (cr2);
+
+	cairo_save (cr);
+	cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
 	cairo_set_source_surface (cr, surface, 0, 0);
+	cairo_paint (cr);
+	cairo_restore (cr);
+
+#if GTK_CHECK_VERSION (3, 0, 0)
+	update_shape_region (surface, windata);
 #else
-	cr = cairo_create(surface);
+	update_shape_mask (windata);
 #endif
 
-	fill_background(widget, windata, cr);
-	draw_border(widget, windata, cr);
-	draw_stripe(widget, windata, cr);
+	cairo_surface_destroy (surface);
+}
 
-#if !GTK_CHECK_VERSION (3, 0, 0)
-	cairo_destroy(cr);
-#endif
-	cairo_set_source_surface(context, surface, 0, 0);
-	cairo_paint(context);
-	cairo_surface_destroy(surface);
-	cairo_destroy(context);
+static gboolean
+#if GTK_CHECK_VERSION (3, 0, 0)
+on_draw (GtkWidget *widget, cairo_t *cr, WindowData *windata)
+{
+	paint_window (widget, cr, windata);
 
 	return FALSE;
 }
+#else
+on_expose_event (GtkWidget *widget, GdkEventExpose *event, WindowData *windata)
+{
+	cairo_t *cr = gdk_cairo_create (event->window);
+
+	paint_window (widget, cr, windata);
+
+	cairo_destroy (cr);
+
+	return FALSE;
+}
+#endif
 
 /* Event handlers */
 static gboolean
@@ -616,50 +714,56 @@ configure_event_cb(GtkWidget *nw,
 	return FALSE;
 }
 
+static void on_composited_changed (GtkWidget* window, WindowData* windata)
+{
+	windata->composited = gdk_screen_is_composited (gtk_widget_get_screen(window));
+
+	gtk_widget_queue_draw (window);
+}
+
 static gboolean
 countdown_expose_cb(GtkWidget *pie,
 #if GTK_CHECK_VERSION (3, 0, 0)
-					cairo_t *cr,
+		    cairo_t *cr,
 #else
-					GdkEventExpose *event,
+		    GdkEventExpose *event,
 #endif
-					WindowData *windata)
+		    WindowData *windata)
 {
-	cairo_t *context;
+#if !GTK_CHECK_VERSION(3, 0, 0)
+	cairo_t *cr = gdk_cairo_create (event->window);
+#endif
+	cairo_t *cr2;
 	cairo_surface_t *surface;
-
-	context = gdk_cairo_create(gtk_widget_get_window(pie));
 	GtkAllocation alloc;
-	gtk_widget_get_allocation(pie, &alloc);
-#if !GTK_CHECK_VERSION(3, 0, 0)
-	cairo_t *cr;
-#endif
 
-	cairo_set_operator(context, CAIRO_OPERATOR_SOURCE);
-	surface = cairo_surface_create_similar(cairo_get_target(context),
-										   CAIRO_CONTENT_COLOR_ALPHA,
-										   alloc.width,
-										   alloc.height);
-#if GTK_CHECK_VERSION(3, 0, 0)
+	cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+
+	gtk_widget_get_allocation (pie, &alloc);
+
+	surface = cairo_surface_create_similar (cairo_get_target (cr),
+					        CAIRO_CONTENT_COLOR_ALPHA,
+						alloc.width,
+						alloc.height);
+	cr2 = cairo_create (surface);
+
+	cairo_translate (cr2, -alloc.x, -alloc.y);
+	fill_background (pie, windata, cr2);
+	cairo_translate (cr2, alloc.x, alloc.y);
+	draw_pie (pie, windata, cr2);
+	cairo_fill (cr2);
+
+	cairo_destroy (cr2);
+
+	cairo_save (cr);
 	cairo_set_source_surface (cr, surface, 0, 0);
-#else
-	cr = cairo_create(surface);
-#endif
-
-
-	cairo_translate (cr, -alloc.x, -alloc.y);
-	fill_background (pie, windata, cr);
-	cairo_translate (cr, alloc.x, alloc.y);
-	
-	draw_pie (pie, windata, cr);
+	cairo_paint (cr);
+	cairo_restore (cr);
 
+	cairo_surface_destroy (surface);
 #if !GTK_CHECK_VERSION(3, 0, 0)
-	cairo_destroy(cr);
+	cairo_destroy (cr);
 #endif
-	cairo_set_source_surface(context, surface, 0, 0);
-	cairo_paint(context);
-	cairo_surface_destroy(surface);
-	cairo_destroy(context);
 	return TRUE;
 }
 
@@ -706,7 +810,6 @@ create_notification(UrlClickedCb url_clicked)
 {
 	GtkWidget *spacer;
 	GtkWidget *win;
-	GtkWidget *drawbox;
 	GtkWidget *main_vbox;
 	GtkWidget *hbox;
 	GtkWidget *vbox;
@@ -727,9 +830,10 @@ create_notification(UrlClickedCb url_clicked)
 	windata->url_clicked = url_clicked;
 
 	win = gtk_window_new(GTK_WINDOW_POPUP);
+	gtk_window_set_resizable(GTK_WINDOW(win), FALSE);
 	windata->win = win;
 
-	windata->enable_transparency = FALSE;
+	windata->composited = FALSE;
 	screen = gtk_window_get_screen(GTK_WINDOW(win));
 #if GTK_CHECK_VERSION(3, 0, 0)
 	visual = gdk_screen_get_rgba_visual(screen);
@@ -738,7 +842,7 @@ create_notification(UrlClickedCb url_clicked)
 	{
 		gtk_widget_set_visual(win, visual);
 		if (gdk_screen_is_composited(screen))
-			windata->enable_transparency = TRUE;
+			windata->composited = TRUE;
 	}
 #else
 	colormap = gdk_screen_get_rgba_colormap(screen);
@@ -747,7 +851,7 @@ create_notification(UrlClickedCb url_clicked)
 	{
 		gtk_widget_set_colormap(win, colormap);
 		if (gdk_screen_is_composited(screen))
-			windata->enable_transparency = TRUE;
+			windata->composited = TRUE;
 	}
 #endif
 
@@ -763,28 +867,19 @@ create_notification(UrlClickedCb url_clicked)
 	g_signal_connect(G_OBJECT(win), "configure_event",
 					 G_CALLBACK(configure_event_cb), windata);
 
-	/*
-	 * For some reason, there are occasionally graphics glitches when
-	 * repainting the window. Despite filling the window with a background
-	 * color, parts of the other windows on the screen or the shadows around
-	 * notifications will appear on the notification. Somehow, adding this
-	 * eventbox makes that problem just go away. Whatever works for now.
-	 */
-	drawbox = gtk_event_box_new();
-	gtk_widget_show(drawbox);
-	gtk_container_add(GTK_CONTAINER(win), drawbox);
+	g_signal_connect (G_OBJECT (win), "composited-changed", G_CALLBACK (on_composited_changed), windata);
 
 	main_vbox = gtk_vbox_new(FALSE, 0);
 	gtk_widget_show(main_vbox);
-	gtk_container_add(GTK_CONTAINER(drawbox), main_vbox);
+	gtk_container_add(GTK_CONTAINER(win), main_vbox);
+#if !GTK_CHECK_VERSION (3, 0, 0)
 	gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 1);
+#endif
 
 #if GTK_CHECK_VERSION (3, 0, 0)
-	g_signal_connect(G_OBJECT(main_vbox), "draw",
-					 G_CALLBACK(paint_window), windata);
+	g_signal_connect (G_OBJECT (main_vbox), "draw", G_CALLBACK (on_draw), windata);
 #else
-	g_signal_connect(G_OBJECT(main_vbox), "expose_event",
-					 G_CALLBACK(paint_window), windata);
+	g_signal_connect (G_OBJECT (main_vbox), "expose_event", G_CALLBACK (on_expose_event), windata);
 #endif
 
 	windata->top_spacer = gtk_image_new();
@@ -820,11 +915,14 @@ create_notification(UrlClickedCb url_clicked)
 	windata->summary_label = gtk_label_new(NULL);
 	gtk_widget_show(windata->summary_label);
 	gtk_box_pack_start(GTK_BOX(hbox), windata->summary_label, TRUE, TRUE, 0);
+#if GTK_CHECK_VERSION (3, 16, 0)
+	gtk_label_set_xalign (GTK_LABEL (windata->summary_label), 0.0);
+	gtk_label_set_yalign (GTK_LABEL (windata->summary_label), 0.0);
+#else
 	gtk_misc_set_alignment(GTK_MISC(windata->summary_label), 0, 0);
-	gtk_label_set_line_wrap(GTK_LABEL(windata->summary_label), TRUE);
-#if GTK_CHECK_VERSION (3, 0, 0)
-	gtk_label_set_line_wrap_mode (GTK_LABEL (windata->summary_label), PANGO_WRAP_WORD_CHAR);
 #endif
+	gtk_label_set_line_wrap(GTK_LABEL(windata->summary_label), TRUE);
+	gtk_label_set_line_wrap_mode (GTK_LABEL (windata->summary_label), PANGO_WRAP_WORD_CHAR);
 
 	atkobj = gtk_widget_get_accessible(windata->summary_label);
 	atk_object_set_description(atkobj, "Notification summary text.");
@@ -845,7 +943,11 @@ create_notification(UrlClickedCb url_clicked)
 	atk_object_set_name(atkobj, "");
 	atk_object_set_description(atkobj, "Closes the notification.");
 
+#if GTK_CHECK_VERSION (3, 10, 0)
+	image = gtk_image_new_from_icon_name ("window-close", GTK_ICON_SIZE_MENU);
+#else
 	image = gtk_image_new_from_stock(GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
+#endif
 	gtk_widget_show(image);
 	gtk_container_add(GTK_CONTAINER(close_button), image);
 
@@ -861,7 +963,12 @@ create_notification(UrlClickedCb url_clicked)
 	windata->icon = gtk_image_new();
 	gtk_box_pack_start(GTK_BOX(windata->iconbox), windata->icon,
 					   TRUE, TRUE, 0);
-	gtk_misc_set_alignment(GTK_MISC(windata->icon), 0.5, 0.0);
+#if GTK_CHECK_VERSION (3, 16, 0)
+		gtk_widget_set_halign (image, GTK_ALIGN_CENTER);
+		gtk_widget_set_valign (image, GTK_ALIGN_CENTER);
+#else
+		gtk_misc_set_alignment(GTK_MISC(image), 0.5, 0.5);
+#endif
 
 	vbox = gtk_vbox_new(FALSE, 6);
 	gtk_widget_show(vbox);
@@ -869,11 +976,15 @@ create_notification(UrlClickedCb url_clicked)
 
 	windata->body_label = gtk_label_new(NULL);
 	gtk_box_pack_start(GTK_BOX(vbox), windata->body_label, TRUE, TRUE, 0);
+#if GTK_CHECK_VERSION (3, 16, 0)
+	gtk_label_set_xalign (GTK_LABEL (windata->body_label), 0.0);
+	gtk_label_set_yalign (GTK_LABEL (windata->body_label), 0.0);
+#else
 	gtk_misc_set_alignment(GTK_MISC(windata->body_label), 0, 0);
-	gtk_label_set_line_wrap(GTK_LABEL(windata->body_label), TRUE);
-#if GTK_CHECK_VERSION (3, 0, 0)
-	gtk_label_set_line_wrap_mode (GTK_LABEL (windata->body_label), PANGO_WRAP_WORD_CHAR);
 #endif
+	gtk_label_set_line_wrap(GTK_LABEL(windata->body_label), TRUE);
+	gtk_label_set_line_wrap_mode (GTK_LABEL (windata->body_label), PANGO_WRAP_WORD_CHAR);
+
 	g_signal_connect(G_OBJECT(windata->body_label), "activate-link",
                          G_CALLBACK(activate_link), windata);
 
@@ -898,11 +1009,15 @@ set_notification_text(GtkWindow *nw, const char *summary, const char *body)
 	WindowData *windata = g_object_get_data(G_OBJECT(nw), "windata");
 	g_assert(windata != NULL);
 
-	str = g_strdup_printf("<b><big>%s</big></b>", summary);
+	str = g_strdup_printf(
+		"<span color=\"#000000\"><b><big>%s</big></b></span>", summary);
 	gtk_label_set_markup(GTK_LABEL(windata->summary_label), str);
 	g_free(str);
 
-	gtk_label_set_markup(GTK_LABEL(windata->body_label), body);
+	str = g_strdup_printf(
+        "<span color=\"#000000\">%s</span>", body);
+	gtk_label_set_markup(GTK_LABEL(windata->body_label), str);
+	g_free(str);
 
 	if (body == NULL || *body == '\0')
 		gtk_widget_hide(windata->body_label);
@@ -1019,13 +1134,22 @@ add_notification_action(GtkWindow *nw, const char *text, const char *key,
 		GtkWidget *image = gtk_image_new_from_pixbuf(pixbuf);
 		gtk_widget_show(image);
 		gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 0);
+#if GTK_CHECK_VERSION (3, 16, 0)
+		gtk_widget_set_halign (image, GTK_ALIGN_CENTER);
+		gtk_widget_set_valign (image, GTK_ALIGN_CENTER);
+#else
 		gtk_misc_set_alignment(GTK_MISC(image), 0.5, 0.5);
+#endif
 	}
 
 	label = gtk_label_new(NULL);
 	gtk_widget_show(label);
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+#if GTK_CHECK_VERSION (3, 16, 0)
+	gtk_label_set_xalign (GTK_LABEL (label), 0.0);
+#else
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+#endif
 	buf = g_strdup_printf("<small>%s</small>", text);
 	gtk_label_set_markup(GTK_LABEL(label), buf);
 	g_free(buf);
