@@ -61,6 +61,24 @@ enum {
 	URGENCY_CRITICAL
 };
 
+gboolean theme_check_init(unsigned int major_ver, unsigned int minor_ver,
+			  unsigned int micro_ver);
+void get_theme_info(char **theme_name, char **theme_ver, char **author,
+		    char **homepage);
+GtkWindow* create_notification(UrlClickedCb url_clicked);
+void set_notification_text(GtkWindow *nw, const char *summary,
+			   const char *body);
+void set_notification_icon(GtkWindow *nw, GdkPixbuf *pixbuf);
+void set_notification_arrow(GtkWidget *nw, gboolean visible, int x, int y);
+void add_notification_action(GtkWindow *nw, const char *text, const char *key,
+			     ActionInvokedCb cb);
+void clear_notification_actions(GtkWindow *nw);
+void move_notification(GtkWidget *nw, int x, int y);
+void set_notification_timeout(GtkWindow *nw, glong timeout);
+void set_notification_hints(GtkWindow *nw, GHashTable *hints);
+void notification_tick(GtkWindow *nw, glong remaining);
+gboolean get_always_stack(GtkWidget* nw);
+
 #define WIDTH          400
 #if GTK_CHECK_VERSION(3, 0, 0)
 #define HEIGHT         100
@@ -77,6 +95,11 @@ enum {
 
 #define MAX_ICON_SIZE IMAGE_SIZE
 
+#if GTK_CHECK_VERSION (3, 2, 0)
+#define gtk_hbox_new(X,Y) gtk_box_new(GTK_ORIENTATION_HORIZONTAL,Y)
+#define gtk_vbox_new(X,Y) gtk_box_new(GTK_ORIENTATION_VERTICAL,Y)
+#endif
+
 static void draw_round_rect(cairo_t* cr, gdouble aspect, gdouble x, gdouble y, gdouble corner_radius, gdouble width, gdouble height)
 {
 	gdouble radius = corner_radius / aspect;
@@ -175,15 +198,45 @@ static void fill_background(GtkWidget* widget, WindowData* windata, cairo_t* cr)
 #endif
 }
 
-static void update_shape(WindowData* windata)
-{
-	cairo_t* cr;
 #if GTK_CHECK_VERSION (3, 0, 0)
-	cairo_surface_t *surface;
-	cairo_region_t *region;
+static void
+update_shape_region (cairo_surface_t *surface,
+		     WindowData      *windata)
+{
+	if (windata->width == windata->last_width && windata->height == windata->last_height)
+	{
+		return;
+	}
+
+	if (windata->width == 0 || windata->height == 0)
+	{
+		GtkAllocation allocation;
+		gtk_widget_get_allocation (windata->win, &allocation);
+
+		windata->width = MAX (allocation.width, 1);
+		windata->height = MAX (allocation.height, 1);
+	}
+
+	if (!windata->composited) {
+		cairo_region_t *region;
+
+		region = gdk_cairo_region_create_from_surface (surface);
+		gtk_widget_shape_combine_region (windata->win, region);
+		cairo_region_destroy (region);
+	} else {
+		gtk_widget_shape_combine_region (windata->win, NULL);
+		return;
+	}
+
+	windata->last_width = windata->width;
+	windata->last_height = windata->height;
+}
 #else
+static void
+update_shape_mask (WindowData* windata)
+{
+	cairo_t *cr;
 	GdkBitmap* mask;
-#endif
 
 	if (windata->width == windata->last_width && windata->height == windata->last_height)
 	{
@@ -192,104 +245,95 @@ static void update_shape(WindowData* windata)
 
 	if (windata->width == 0 || windata->height == 0)
 	{
-			GtkAllocation allocation;
+		GtkAllocation allocation;
 
-			gtk_widget_get_allocation(windata->win, &allocation);
+		gtk_widget_get_allocation (windata->win, &allocation);
 
-			windata->width = MAX(allocation.width, 1);
-			windata->height = MAX(allocation.height, 1);
+		windata->width = MAX (allocation.width, 1);
+		windata->height = MAX (allocation.height, 1);
 	}
 
 	if (windata->composited)
 	{
-#if GTK_CHECK_VERSION (3, 0, 0)
-		gtk_widget_shape_combine_region (windata->win, NULL);
-#else
-		gtk_widget_shape_combine_mask(windata->win, NULL, 0, 0);
-#endif
+		gtk_widget_shape_combine_mask (windata->win, NULL, 0, 0);
 		return;
 	}
 
 	windata->last_width = windata->width;
 	windata->last_height = windata->height;
-#if GTK_CHECK_VERSION (3, 0, 0)
-	surface = cairo_image_surface_create (CAIRO_FORMAT_A8, windata->width, windata->height);
-	cr = cairo_create (surface);
-#else
-	mask = (GdkBitmap*) gdk_pixmap_new(NULL, windata->width, windata->height, 1);
+
+	mask = (GdkBitmap*) gdk_pixmap_new (NULL, windata->width, windata->height, 1);
 
 	if (mask == NULL)
 	{
 		return;
 	}
-	cr = gdk_cairo_create(mask);
-#endif
+	cr = gdk_cairo_create (mask);
 
-	if (cairo_status(cr) == CAIRO_STATUS_SUCCESS)
+	if (cairo_status (cr) == CAIRO_STATUS_SUCCESS)
 	{
-		cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
-		cairo_paint(cr);
+		cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
+		cairo_paint (cr);
 
-		cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
-		cairo_set_source_rgb(cr, 1.0f, 1.0f, 1.0f);
-		draw_round_rect(cr, 1.0f, DEFAULT_X0, DEFAULT_Y0, DEFAULT_RADIUS, windata->width, windata->height);
-		cairo_fill(cr);
+		cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+		cairo_set_source_rgb (cr, 1.0f, 1.0f, 1.0f);
+		draw_round_rect (cr, 1.0f, DEFAULT_X0, DEFAULT_Y0, DEFAULT_RADIUS, windata->width, windata->height);
+		cairo_fill (cr);
 
-#if GTK_CHECK_VERSION (3, 0, 0)
-		region = gdk_cairo_region_create_from_surface (surface);
-		gtk_widget_shape_combine_region (windata->win, region);
-		cairo_region_destroy (region);
-#else
-		gtk_widget_shape_combine_mask(windata->win, mask, 0, 0);
-#endif
+		gtk_widget_shape_combine_mask (windata->win, mask, 0, 0);
 	}
-	cairo_destroy(cr);
 
-#if GTK_CHECK_VERSION (3, 0, 0)
-	cairo_surface_destroy (surface);
-#else
-	g_object_unref(mask);
-#endif
+	cairo_destroy (cr);
+	g_object_unref (mask);
 }
-
-static void paint_window(GtkWidget* widget, WindowData* windata)
+#endif
+static void paint_window (GtkWidget  *widget,
+			  cairo_t    *cr,
+			  WindowData *windata)
 {
-	cairo_t* context;
-	cairo_surface_t* surface;
-	cairo_t* cr;
+	cairo_surface_t *surface;
+	cairo_t *cr2;
 
 	if (windata->width == 0 || windata->height == 0)
 	{
-			GtkAllocation allocation;
-
-			gtk_widget_get_allocation(windata->win, &allocation);
-
-			windata->width = MAX(allocation.width, 1);
-			windata->height = MAX(allocation.height, 1);
-	}
-
-	context = gdk_cairo_create(gtk_widget_get_window(widget));
-
-	cairo_set_operator(context, CAIRO_OPERATOR_SOURCE);
-
-
 		GtkAllocation allocation;
 
-		gtk_widget_get_allocation(widget, &allocation);
+		gtk_widget_get_allocation (windata->win, &allocation);
 
-		surface = cairo_surface_create_similar(cairo_get_target(context), CAIRO_CONTENT_COLOR_ALPHA, allocation.width, allocation.height);
+		windata->width = MAX (allocation.width, 1);
+		windata->height = MAX (allocation.height, 1);
+	}
 
-    cr = cairo_create(surface);
+	cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
 
-	fill_background(widget, windata, cr);
 
-	cairo_destroy(cr);
-	cairo_set_source_surface(context, surface, 0, 0);
-	cairo_paint(context);
+	surface = cairo_surface_create_similar (cairo_get_target (cr),
+						CAIRO_CONTENT_COLOR_ALPHA,
+						windata->width,
+						windata->height);
+
+	cr2 = cairo_create (surface);
+
+	/* transparent background */
+	cairo_rectangle (cr2, 0, 0, windata->width, windata->height);
+	cairo_set_source_rgba (cr2, 0.0, 0.0, 0.0, 0.0);
+	cairo_fill (cr2);
+
+	fill_background (widget, windata, cr2);
+
+	cairo_destroy(cr2);
+
+	cairo_save (cr);
+	cairo_set_source_surface(cr, surface, 0, 0);
+	cairo_paint(cr);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	update_shape_region (surface, windata);
+#else
+	update_shape_mask (windata);
+#endif
+	cairo_restore (cr);
+
 	cairo_surface_destroy(surface);
-	cairo_destroy(context);
-
-	update_shape(windata);
 }
 
 static gboolean on_window_map(GtkWidget* widget, GdkEvent* event, WindowData* windata)
@@ -298,15 +342,30 @@ static gboolean on_window_map(GtkWidget* widget, GdkEvent* event, WindowData* wi
 }
 
 #if GTK_CHECK_VERSION (3, 0, 0)
-static gboolean on_window_draw(GtkWidget* widget, cairo_t* cr, WindowData* windata)
-#else
-static gboolean on_window_expose(GtkWidget* widget, GdkEventExpose* event, WindowData* windata)
-#endif
+static gboolean
+on_draw (GtkWidget  *widget,
+	 cairo_t    *cr,
+	 WindowData *windata)
 {
-	paint_window(widget, windata);
+	paint_window (widget, cr, windata);
 
 	return FALSE;
 }
+#else
+static gboolean
+on_expose_event (GtkWidget      *widget,
+		 GdkEventExpose *event,
+		 WindowData     *windata)
+{
+	cairo_t *cr = gdk_cairo_create (event->window);
+
+	paint_window (widget, cr, windata);
+
+	cairo_destroy (cr);
+
+	return FALSE;
+}
+#endif
 
 static void destroy_windata(WindowData* windata)
 {
@@ -519,7 +578,8 @@ static void on_style_set(GtkWidget* widget, GtkStyle* previous_style, WindowData
 static void on_composited_changed(GtkWidget* window, WindowData* windata)
 {
 	windata->composited = gdk_screen_is_composited(gtk_widget_get_screen(window));
-	update_shape(windata);
+
+	gtk_widget_queue_draw (windata->win);
 }
 
 GtkWindow* create_notification(UrlClickedCb url_clicked)
@@ -531,7 +591,9 @@ GtkWindow* create_notification(UrlClickedCb url_clicked)
 	GtkWidget* image;
 	GtkWidget* alignment;
 	AtkObject* atkobj;
+#if !GTK_CHECK_VERSION(3, 0, 0)
 	GtkRcStyle* rcstyle;
+#endif
 	WindowData* windata;
 #if GTK_CHECK_VERSION(3, 0, 0)
 	GdkVisual *visual;
@@ -554,9 +616,9 @@ GtkWindow* create_notification(UrlClickedCb url_clicked)
 #endif
 	g_signal_connect(G_OBJECT(win), "map-event", G_CALLBACK(on_window_map), windata);
 #if GTK_CHECK_VERSION (3, 0, 0)
-	g_signal_connect(G_OBJECT(win), "draw", G_CALLBACK(on_window_draw), windata);
+	g_signal_connect(G_OBJECT(win), "draw", G_CALLBACK(on_draw), windata);
 #else
-	g_signal_connect(G_OBJECT(win), "expose-event", G_CALLBACK(on_window_expose), windata);
+	g_signal_connect(G_OBJECT(win), "expose-event", G_CALLBACK(on_expose_event), windata);
 #endif
 	g_signal_connect(G_OBJECT(win), "realize", G_CALLBACK(on_window_realize), windata);
 
@@ -649,17 +711,23 @@ GtkWindow* create_notification(UrlClickedCb url_clicked)
 	gtk_container_set_border_width(GTK_CONTAINER(close_button), 0);
 	g_signal_connect_swapped(G_OBJECT(close_button), "clicked", G_CALLBACK(gtk_widget_destroy), win);
 
+#if !GTK_CHECK_VERSION(3, 0, 0)
 	rcstyle = gtk_rc_style_new();
 	rcstyle->xthickness = rcstyle->ythickness = 0;
 	gtk_widget_modify_style(close_button, rcstyle);
 	g_object_unref(rcstyle);
+#endif
 
 	atkobj = gtk_widget_get_accessible(close_button);
 	atk_action_set_description(ATK_ACTION(atkobj), 0, "Closes the notification.");
 	atk_object_set_name(atkobj, "");
 	atk_object_set_description(atkobj, "Closes the notification.");
 
+#if GTK_CHECK_VERSION (3, 10, 0)
+	image = gtk_image_new_from_icon_name ("window-close", GTK_ICON_SIZE_MENU);
+#else
 	image = gtk_image_new_from_stock(GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
+#endif
 	gtk_widget_show(image);
 	gtk_container_add(GTK_CONTAINER(close_button), image);
 
@@ -672,7 +740,12 @@ GtkWindow* create_notification(UrlClickedCb url_clicked)
 #endif
 	gtk_widget_show(windata->summary_label);
 	gtk_box_pack_start(GTK_BOX(vbox), windata->summary_label, TRUE, TRUE, 0);
+#if GTK_CHECK_VERSION (3, 16, 0)
+	gtk_label_set_xalign (GTK_LABEL (windata->summary_label), 0.0);
+	gtk_label_set_yalign (GTK_LABEL (windata->summary_label), 0.0);
+#else
 	gtk_misc_set_alignment(GTK_MISC(windata->summary_label), 0, 0);
+#endif
 	gtk_label_set_line_wrap(GTK_LABEL(windata->summary_label), TRUE);
 #if GTK_CHECK_VERSION (3, 0, 0)
 	gtk_label_set_line_wrap_mode (GTK_LABEL (windata->summary_label), PANGO_WRAP_WORD_CHAR);
@@ -698,7 +771,12 @@ GtkWindow* create_notification(UrlClickedCb url_clicked)
 #endif
 	gtk_widget_show(windata->body_label);
 	gtk_box_pack_start(GTK_BOX(vbox), windata->body_label, TRUE, TRUE, 0);
+#if GTK_CHECK_VERSION (3, 16, 0)
+	gtk_label_set_xalign (GTK_LABEL (windata->body_label), 0.0);
+	gtk_label_set_yalign (GTK_LABEL (windata->body_label), 0.0);
+#else
 	gtk_misc_set_alignment(GTK_MISC(windata->body_label), 0, 0);
+#endif
 	gtk_label_set_line_wrap(GTK_LABEL(windata->body_label), TRUE);
 #if GTK_CHECK_VERSION (3, 0, 0)
 	gtk_label_set_line_wrap_mode (GTK_LABEL (windata->body_label), PANGO_WRAP_WORD_CHAR);
@@ -795,7 +873,11 @@ void set_notification_text(GtkWindow* nw, const char* summary, const char* body)
 
 	update_content_hbox_visibility(windata);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_widget_get_preferred_size (windata->close_button, NULL, &req);
+#else
 	gtk_widget_size_request(windata->close_button, &req);
+#endif
 	/* -1: main_vbox border width
 	   -10: vbox border width
 	   -6: spacing for hbox */
@@ -927,8 +1009,8 @@ static gboolean on_countdown_expose(GtkWidget* pie, GdkEventExpose* event, Windo
 
 	cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
 #if GTK_CHECK_VERSION (3, 0, 0)
-	gtk_style_context_get_color (context, GTK_STATE_FLAG_NORMAL, &color);
-	gtk_style_context_get_background_color (context, GTK_STATE_FLAG_NORMAL, &fg_color);
+	gtk_style_context_get_color (style, GTK_STATE_FLAG_NORMAL, &color);
+	gtk_style_context_get_background_color (style, GTK_STATE_FLAG_NORMAL, &fg_color);
 	r = color.red;
 	g = color.green;
 	b = color.blue;
@@ -1044,7 +1126,12 @@ void add_notification_action(GtkWindow* nw, const char* text, const char* key, A
 #endif
 		gtk_widget_show(image);
 		gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 0);
+#if GTK_CHECK_VERSION (3, 16, 0)
+		gtk_widget_set_halign (image, GTK_ALIGN_CENTER);
+		gtk_widget_set_valign (image, GTK_ALIGN_CENTER);
+#else
 		gtk_misc_set_alignment(GTK_MISC(image), 0.5, 0.5);
+#endif
 	}
 
 	label = gtk_label_new(NULL);
@@ -1055,7 +1142,11 @@ void add_notification_action(GtkWindow* nw, const char* text, const char* key, A
 #endif
 	gtk_widget_show(label);
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+#if GTK_CHECK_VERSION (3, 16, 0)
+	gtk_label_set_xalign (GTK_LABEL (label), 0.0);
+#else
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+#endif
 	buf = g_strdup_printf("<small>%s</small>", text);
 	gtk_label_set_markup(GTK_LABEL(label), buf);
 	g_free(buf);
